Module: ProductTaxonomy::LocalizationsValidator
- Defined in:
- lib/product_taxonomy/localizations_validator.rb
Class Method Summary collapse
-
.validate!(locales = nil) ⇒ Object
Validate that all localizations are present for the given locales.
Class Method Details
.validate!(locales = nil) ⇒ Object
Validate that all localizations are present for the given locales. If no locales are provided, all locales will be validated and the consistency of locales across models will be checked.
10 11 12 13 14 15 16 17 |
# File 'lib/product_taxonomy/localizations_validator.rb', line 10 def validate!(locales = nil) Category.validate_localizations!(locales) Attribute.validate_localizations!(locales) Value.validate_localizations!(locales) ReturnReason.validate_localizations!(locales) validate_locales_are_consistent! if locales.nil? end |